/**
 * Copyright 1998-2007, CHISEL Group, University of Victoria, Victoria, BC, Canada.
 * All rights reserved.
 */
package ca.uvic.csr.shrimp.gui.QueryView;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import java.util.Vector;

import ca.uvic.csr.shrimp.ShrimpConstants;
import ca.uvic.csr.shrimp.DataBean.DataBean;
import ca.uvic.csr.shrimp.DataDisplayBridge.DataDisplayBridge;
import ca.uvic.csr.shrimp.DisplayBean.DisplayBean;
import ca.uvic.csr.shrimp.DisplayBean.ShrimpNode;

/**
 * 
 * @author Chris Callendar
 * @date 15-Dec-06
 */
class ExpandAction extends QueryViewMenuAction {

	public static final String EXPAND = ShrimpConstants.ACTION_NAME_EXPAND;

	public ExpandAction(QueryView queryView) {
		super(EXPAND, null, queryView);
	}

	protected void doAction(DisplayBean displayBean, ShrimpNode node) {
		DataBean dataBean = getDataBean();
		if (dataBean == null) {
			return;
		}
		DataDisplayBridge dataDisplayBridge = displayBean.getDataDisplayBridge();
		Collection artTypes = queryView.getArtTypes();
		Collection relTypes = queryView.getRelTypes();
		List srcArtifacts = new ArrayList(1);
		srcArtifacts.add(node.getArtifact());
		
		// use incoming/outgoing levels of 2 - this will get the immediate neighbors including the node
		Collection connectedArts = dataBean.getConnectedArtifacts(srcArtifacts, artTypes, relTypes, 2, 2);
		connectedArts.remove(node.getArtifact());
		if (!connectedArts.isEmpty()) {
			Vector connectedNodes = dataDisplayBridge.getShrimpNodes(new Vector(connectedArts), true);
			Vector nodesAlreadyInDisplay = displayBean.getAllNodes();
			for (Iterator iter = connectedNodes.iterator(); iter.hasNext();) {
				ShrimpNode connectedNode = (ShrimpNode) iter.next();
				if (nodesAlreadyInDisplay.contains(connectedNode)) {
					iter.remove();
				}
			}
			if (!connectedNodes.isEmpty()) {
				//node.setIsCollapsed(false);
				for (Iterator iter = connectedNodes.iterator(); iter.hasNext();) {
					ShrimpNode connectedNode = (ShrimpNode) iter.next();
					//connectedNode.setIsCollapsed(true);
					displayBean.addObject(connectedNode);
					displayBean.setTransformOf(connectedNode, displayBean.getTransformOf(node));
					displayBean.setVisible(connectedNode, true, false);
				}
				
				// re-layout the new nodes
				displayBean.setLayoutMode(displayBean.getVisibleNodes(), queryView.getQueryHelper().getLayoutMode(), false, true);
			}
		}
	}
	
}